@@ -2,6 +2,27 @@
 # Maarten Speekenbrink 23-3-2008
 # 
 
+rdirichlet <- function(n, alpha) {
+  # taken from gtools...
+    l <- length(alpha)
+    x <- matrix(rgamma(l * n, alpha), ncol = l, byrow = TRUE)
+    sm <- x %*% rep(1, l)
+    x/as.vector(sm)
+}
+
+which.is.max <- function(x) {
+    # taken from MASS
+    y <- seq_along(x)[x == max(x)]
+    if(length(y) > 1L) sample(y, 1L) else y
+}
+
+
+ind.max <- function(x) {
+    out <- rep(0,length(x))
+    out[which.is.max(x)] <- 1
+    out
+}
+
 em <- function(object,...) {
 	if(!is(object,"mix")) stop("object is not of class '(dep)mix'")
 	call <- match.call()
@@ -15,7 +36,9 @@ em <- function(object,...) {
 }
 
 # em for lca and mixture models
-em.mix <- function(object,maxit=100,tol=1e-8,crit="relative",random.start=TRUE,verbose=FALSE,...) {
+em.mix <- function(object,maxit=100,tol=1e-8,crit="relative",random.start=TRUE,verbose=FALSE,classification=c("soft","hard"),...) {
+	
+	clsf <- match.arg(classification)
 	
 	if(!is(object,"mix")) stop("object is not of class 'mix'")
 		
@@ -31,13 +54,17 @@ em.mix <- function(object,maxit=100,tol=1e-8,crit="relative",random.start=TRUE,v
 	if(random.start) {
 				
 		nr <- sum(ntimes(object))
-		gamma <- matrix(runif(nr*ns,min=.0001,max=.9999),nrow=nr,ncol=ns)
-		gamma <- gamma/rowSums(gamma)
+		gamma <- rdirichlet(nr,alpha=rep(.01,ns))
+		
+		if(clsf == "hard") {
+		    gamma <- t(apply(gamma,1,ind.max))
+		}
+
 		LL <- -1e10
 		
 		for(i in 1:ns) {
 			for(k in 1:nresp(object)) {
-				object@response[[i]][[k]] <- fit(object@response[[i]][[k]],w=gamma[,i])
+			    object@response[[i]][[k]] <- fit(object@response[[i]][[k]],w=gamma[,i])
 				# update dens slot of the model
 				object@dens[,k,i] <- dens(object@response[[i]][[k]])
 			}
@@ -65,6 +92,10 @@ em.mix <- function(object,maxit=100,tol=1e-8,crit="relative",random.start=TRUE,v
 		# maximization
 		
 		# should become object@prior <- fit(object@prior)
+		
+		if(clsf == "hard") {
+		    gamma <- t(apply(gamma,1,ind.max))
+		}
 		object@prior@y <- gamma[bt,,drop=FALSE]
 		object@prior <- fit(object@prior, w=NULL,ntimes=NULL)
 		object@init <- dens(object@prior)
@@ -126,10 +157,14 @@ em.mix <- function(object,maxit=100,tol=1e-8,crit="relative",random.start=TRUE,v
 }
 
 # em for hidden markov models
-em.depmix <- function(object,maxit=100,tol=1e-8,crit="relative",random.start=TRUE,verbose=FALSE,...) {
+em.depmix <- function(object,maxit=100,tol=1e-8,crit="relative",random.start=TRUE,verbose=FALSE, classification=c("soft","hard"),...) {
 	
 	if(!is(object,"depmix")) stop("object is not of class 'depmix'")
 	
+	clsf <- match.arg(classification)
+	
+	clsf="soft"
+	
 	ns <- nstates(object)
 	
 	ntimes <- ntimes(object)
@@ -143,8 +178,12 @@ em.depmix <- function(object,maxit=100,tol=1e-8,crit="relative",random.start=TRU
 	if(random.start) {
 				
 		nr <- sum(ntimes(object))
-		gamma <- matrix(runif(nr*ns,min=.0001,max=.9999),nrow=nr,ncol=ns)
-		gamma <- gamma/rowSums(gamma)
+		#gamma <- matrix(runif(nr*ns,min=.0001,max=.9999),nrow=nr,ncol=ns)
+		#gamma <- gamma/rowSums(gamma)
+		gamma <- rdirichlet(nr,alpha=rep(.01,ns))
+		if(clsf == "hard") {
+		    gamma <- t(apply(gamma,1,ind.max))
+		}
 		LL <- -1e10
 		
 		for(i in 1:ns) {
